package com.zteits.irain.portal.web.parkinglotcloudplatform.datastatistic;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import com.alibaba.dubbo.common.utils.StringUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.clouds.common.constants.SessionEnum;
import com.clouds.common.entity.UserInfo;
import com.clouds.common.utils.excle.ExcelUtil;
import com.clouds.common.utils.excle.ExcleFillDateManager;
import com.clouds.common.utils.excle.Layouter;
import com.clouds.common.web.BizController;
import com.clouds.common.web.SessionCommUtil;
import com.zteits.clouds.api.apibase.bean.BizResult;
import com.zteits.clouds.api.apibase.bean.PageBean;
import com.zteits.clouds.api.dto.clouds.dto.BillManageDTO;
import com.zteits.clouds.api.dto.clouds.param.BillQueryRequest;
import com.zteits.clouds.api.dto.pay.param.TdCompanyParkQueryRequest;
import com.zteits.clouds.api.service.clouds.BillManageForDayService;
import com.zteits.clouds.api.service.pay.TdCustCompanyService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 日账单controller类.<br/>
 * 
 * Copyright: Copyright (c) 2017  zteits
 * 
 * @ClassName: BillManageController.java
 * @Description: 
 * @version: v1.0.0
 * @author: wangfs
 * @date: 2017年6月30日   下午1:14:16 
 * Modification History:
 * Date             Author          Version            Description
 *---------------------------------------------------------*
 * 2017年6月30日      wangfs           v1.0.0               创建
 */

@Api(value = "停车场云平台 日账单", description = "停车场云平台 日账单")
@Controller
@RequestMapping("/bill")
public class BillManageController extends BizController {
	private Logger logger = LoggerFactory.getLogger(BillManageController.class);
	@Autowired
	private BillManageForDayService billManageForDayService;

	@Autowired
	private TdCustCompanyService tdCustCompanyService;
	
	@Autowired
	private SessionCommUtil sessionCommUtil;
	
	@Autowired
	private Map<String,TdCustCompanyService> map;

	private SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	@ApiOperation("日账汇总单查询")
	@RequestMapping("/queryBillforDayForPage")
	@ResponseBody
	public void queryBillforDayTotalForPage(@RequestBody BillQueryRequest billQueryRequest, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		TdCustCompanyService sert = map.get("tdCustCompanyService");
		logger.info("---begin--日账汇总单查询调用后场dubbo服务,入参＝{}", JSONObject.toJSON(billQueryRequest));
		System.out.println("seesionId="+request.getSession().getId());
		UserInfo userInfo = sessionCommUtil.getUserInfo();
		logger.info("从session中获取的用户信息={}" + JSONObject.toJSONString(userInfo));
		TdCompanyParkQueryRequest tdCompanyParkQueryRequest = new TdCompanyParkQueryRequest();
		if (userInfo != null) {
			
			tdCompanyParkQueryRequest.setSysCode(billQueryRequest.getSysCode());
			tdCompanyParkQueryRequest.setCustIdList(userInfo.getOrgIds());
			tdCompanyParkQueryRequest.setSessionId(request.getSession().getId());
			billQueryRequest.setParkIdList(this.queryParkNoByCustIds(tdCompanyParkQueryRequest));
		}
		
		BizResult<PageBean<BillManageDTO>> result = billManageForDayService.queryBillforDayTotalForPage(billQueryRequest);
		result.setErrMsg(billQueryRequest.getParkIdList().toString());
		logger.info("---end--日账汇总单查询调用后场dubbo服务,结果＝{}", JSONObject.toJSONString(result));
		this.returnJsonDataGrid(response, result);
	}

	/**
	 * 查询日账单.<br/>
	 * 
	 * @param request
	 * @param response
	 * @throws Exception
	 */
	@ApiOperation("日账单查看")
	@RequestMapping(value = "/queryBillForDay", method = RequestMethod.POST)
	@ResponseBody
	public void queryBillForDay(@RequestBody BillQueryRequest billQueryRequest, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		BizResult<PageBean<BillManageDTO>> result = new BizResult<PageBean<BillManageDTO>>();
		logger.info("---begin---日账单查看调用后场dubbo服务,入参={}", JSONObject.toJSONString(billQueryRequest));
		TdCompanyParkQueryRequest tdCompanyParkQueryRequest = new TdCompanyParkQueryRequest();
		UserInfo userInfo = (UserInfo) request.getSession().getAttribute(SessionEnum.USER_INFO.key());
		logger.info("从session中获取的用户信息={}" + JSONObject.toJSONString(userInfo));
		if (userInfo != null) {
			
			tdCompanyParkQueryRequest.setSysCode(billQueryRequest.getSysCode());
			tdCompanyParkQueryRequest.setCustIdList(userInfo.getOrgIds());
			tdCompanyParkQueryRequest.setSessionId(request.getSession().getId());
			billQueryRequest.setUserName(userInfo.getName());
			billQueryRequest.setParkIdList(this.queryParkNoByCustIds(tdCompanyParkQueryRequest));
			
		}
		result = billManageForDayService.queryBillforDayDeatilPage(billQueryRequest);
		logger.info("---end---日账单查看调用后场dubbo服务,结果＝{}", JSONObject.toJSON(result));

		this.returnJsonDataGrid(response, result);
	}
	
	/**
	 * 获取登录用户信息.<br/>
	 * @param request
	 * @param response
	 * @throws Exception
	 */
	@RequestMapping(value = "/queryUserInfoForName", method = RequestMethod.POST)
	public void queryUserInfoForName( HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		UserInfo userInfo = (UserInfo) request.getSession().getAttribute(SessionEnum.USER_INFO.key());
		this.returnJson(response, userInfo);
	}
	
	
	


	/**
	 * 日账单导出.<br/>
	 * 
	 * @param request
	 * @param response
	 * @throws Exception 
	 */
	@RequestMapping("/exportToExcleForBillForDay")
	public void exportToExcleForBillForDay(String sysCode, String beginTime, String endTime, String aliTotalFee ,
			String aliDealCount,String wxTotalFee,String wxDealCount,String cashTotalFee,String cashDealCount,
			String orderTotalFee,String orderDealCount,
			HttpServletRequest request,String parkIds,String userName,
			HttpServletResponse response) throws Exception {
		
		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		//获取session中的用户信息
		UserInfo userInfo = sessionCommUtil.getUserInfo();
		BillQueryRequest billQueryRequest  = new BillQueryRequest();
		
		billQueryRequest.setBeginTime(format.parse(beginTime));
		billQueryRequest.setEndTime(format.parse(endTime));
		billQueryRequest.setSysCode(sysCode);
		billQueryRequest.getBaseRequest().setPageSize(0);
		List<String> parkList= new ArrayList<String>();
		if(StringUtils.isNotEmpty(parkIds)){
			for(String str :parkIds.split(",")){
				if(StringUtils.isNotEmpty(str)){
					parkList.add(str.trim());
				}
			}
		}
		billQueryRequest.setParkIdList(parkList);
		/**查询日账单汇总信息.*/
		BizResult<PageBean<BillManageDTO>> result = billManageForDayService.queryBillforDayDeatilPage(billQueryRequest);
		String[] titleOne = new String[2];      
		titleOne[0] = "用户: " + (StringUtils.isNotEmpty(userName) ? userName : "");
		titleOne[1] = "时间: "+beginTime+" "+endTime;
		String[] titleTwo = new String[] {"","现金", "微信", "支付宝", "合计"};
		String[] titleThree = new String[] { "停车场名称", "现金", "微信", "支付宝", "合计"};
		String sheetName = "日账单";
		String fileName = "日账单管理" + format.format(new Date());
	    
		
		// 1.创建excel信息
		XSSFSheet workSheet = ExcelUtil.createExcel(sheetName);
		// 2.设置excel表头和表体
		Layouter.buildBillForDay(workSheet, titleOne, 0, 0);//创建第一行标题
		Layouter.buildHeaders(workSheet, titleTwo, 0, 0);//创建第2行标题
		Layouter.buildHeaders(workSheet, titleThree, 3, 0);//创建第3行标题
		// 3.填充数据
		List<Object[]> contentListTwo = new ArrayList<Object[]>();
		for(int i = 0;i<2;i++){
			Object[] str = new Object[titleTwo.length];
			if(i == 0){
				str[0] = "交易笔数";
				str[1] = (cashDealCount == null ? "0" : cashDealCount);
				str[2] = (wxDealCount== null ? "0" : wxDealCount);
				str[3] = (aliDealCount== null ? "0" : aliDealCount);
				str[4] = (orderDealCount== null ? "0" : orderDealCount);
			}else{
				str[0] = "总收入(元)";
				str[1] = (cashTotalFee == null ? "0" : new BigDecimal(cashTotalFee).divide(new BigDecimal("100"),2, BigDecimal.ROUND_HALF_UP)) ;
				str[2] = (wxTotalFee== null ? "0" : new BigDecimal(wxTotalFee).divide(new BigDecimal("100"),2, BigDecimal.ROUND_HALF_UP)) ;
				str[3] = (aliTotalFee== null ? "0" : new BigDecimal(aliTotalFee).divide(new BigDecimal("100"),2, BigDecimal.ROUND_HALF_UP)) ;
				str[4] = (orderTotalFee== null ? "0" : new BigDecimal(orderTotalFee).divide(new BigDecimal("100"),2, BigDecimal.ROUND_HALF_UP)) ;
			}
			contentListTwo.add(str);
		}
		ExcleFillDateManager fillUserManagerTwo = new ExcleFillDateManager();
		fillUserManagerTwo.fillSalesOrga(workSheet, titleTwo, contentListTwo, 2); //第2行标题
		
		//
		List<Object[]> contentList = new ArrayList<Object[]>();
		if(result != null && result.getData() != null){
			List<BillManageDTO> list = result.getData().getDataList();
			if(list != null && list.size()>0){
				for (int i = 0; i < list.size(); i++) {
					BillManageDTO e = list.get(i);
					Object[] obj = new Object[titleThree.length];
					int index = 0;
					obj[index++] = e.getParkName();
					obj[index++] = (e.getCashTotalFee()== null ? "0" : e.getCashTotalFee().divide(new BigDecimal("100"),2, BigDecimal.ROUND_HALF_UP)) ;
					obj[index++] = (e.getWxTotalFee()== null ? "0" :  e.getWxTotalFee().divide(new BigDecimal("100"),2, BigDecimal.ROUND_HALF_UP)) ;
					obj[index++] = (e.getAliTotalFee()== null ? "0" :  e.getAliTotalFee().divide(new BigDecimal("100"),2, BigDecimal.ROUND_HALF_UP)) ;
					obj[index++] = (e.getAliTotalFee()== null ? "0" : e.getAliTotalFee().divide(new BigDecimal("100"),2, BigDecimal.ROUND_HALF_UP)) ;
					contentList.add(obj);
				}
			}
		}
		
		ExcleFillDateManager fillUserManager = new ExcleFillDateManager();
		fillUserManager.fillSalesOrga(workSheet, titleThree, contentList, 5);
		// 4.excel输出配置
		ExcelUtil.write(response, workSheet, fileName);
		
		
	}

	/**
	 * 通过session信息获取停车场编码.<br/>
	 * 
	 * @param billQueryRequest
	 * @param userInfo
	 * @return
	 * @throws Exception
	 */
	private List<String> queryParkNoByCustIds(TdCompanyParkQueryRequest tdCompanyParkQueryRequest) throws Exception {
		/** 查询停车场编码. */
		BizResult<List<String>> parkNoResult = tdCustCompanyService.queryParkNoByCustIds(tdCompanyParkQueryRequest);
		if (parkNoResult.isSuccess() && parkNoResult != null && parkNoResult.getData() != null) {
			logger.info("根据登录获取到对应的停车场编码=" + JSONArray.toJSONString(parkNoResult.getData()));
			return parkNoResult.getData();
		} else {
			logger.info("根据登录用户没有获取到对应的停车场编码");
			return null;
		}

	}
	
	
	
}
