package com.zteits.irain.portal.web.parkinglotcloudplatform.datastatistic;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpSession;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.dubbo.common.utils.CollectionUtils;
import com.clouds.common.cache.park.ParkingLotCacheUtil;
import com.clouds.common.entity.UserInfo;
import com.clouds.common.utils.ResultUtils;
import com.clouds.common.web.SessionCommUtil;
import com.clouds.common.web.vo.BizResultVO;
import com.google.common.collect.Lists;
import com.zteits.clouds.api.apibase.bean.BizResult;
import com.zteits.clouds.api.apibase.constants.ErrorType;
import com.zteits.clouds.api.apibase.exception.BizException;
import com.zteits.clouds.api.dto.park.dto.ParkingLotDTO;
import com.zteits.clouds.api.dto.park.param.ParkFreeBerthGetByPlnoRequest;
import com.zteits.clouds.api.dto.park.param.QueryParkLotInfoByPkNoRequest;
import com.zteits.clouds.api.dto.pay.param.TdCompanyParkQueryRequest;
import com.zteits.clouds.api.service.park.ParkFreeBerthService;
import com.zteits.clouds.api.service.park.ParkingLotQueryService;
import com.zteits.clouds.api.service.pay.TdCustCompanyService;
import com.zteits.irain.portal.vo.govclouds.ParkLotStatisticVO;
import com.zteits.irain.portal.web.parkinglotcloudplatform.parklotmanage.ParkLotManageController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
/**
 * 
 * Copyright: Copyright (c) 2017  zteits
 * 
 * @ClassName: ParkLotStatisticController.java
 * @Description: 
 * @version: v1.0.0
 * @author: zhaowg
 * @date: 2017年8月7日   下午5:21:16 
 * Modification History: 
 * Date             Author          Version            Description
 *---------------------------------------------------------*
 * 2017年8月7日      zhaowg              v1.0.0               创建
 */

@Api("停车场信息统计")
@RestController
@RequestMapping("parklotstatistic")
public class ParkLotStatisticController {
	private Logger logger = LoggerFactory.getLogger(ParkLotManageController.class);
	@Autowired
	private HttpSession session;
	@Autowired
	private SessionCommUtil sessionCommUtil;
	@Autowired
	private TdCustCompanyService tdCustCompanyService;
	@Autowired
	private ParkingLotQueryService parkingLotQueryService;
	@Autowired
	private ParkFreeBerthService parkFreeBerthService;
	
	@ApiOperation("获取停车场信息统计")
	@PostMapping("getParkLotStatistic")
	public BizResultVO<List<ParkLotStatisticVO>> getParkLotStatistic(@RequestParam String sysCode) throws Exception{
		List<ParkLotStatisticVO> result = Lists.newArrayList();
		//拥有的停车场编号
		List<String> plNos = this.GetParkLotNosByCurrUser(sysCode);
		//查询对应的停车场信息
		if(CollectionUtils.isEmpty(plNos)){
			return new BizResultVO<List<ParkLotStatisticVO>>().setData(result);
		}
		//空闲车位数
		ParkFreeBerthGetByPlnoRequest request = new ParkFreeBerthGetByPlnoRequest();
		request.setPlNos(plNos);
		request.setSysCode(sysCode);
		BizResult<Map<String, Integer>> freeBerthsNumBizResult = parkFreeBerthService.getParkFreeBerthByPlno(request );
		Map<String, Integer> freeBerthsNumMap = ResultUtils.getBizResultData(freeBerthsNumBizResult);
		for (String plNo : plNos) {
			//先查询缓存
			ParkingLotDTO lotDTO = ParkingLotCacheUtil.getParkLotByPlNo(plNo);
			if(lotDTO == null){
				//查询数据库
				QueryParkLotInfoByPkNoRequest request1 = new QueryParkLotInfoByPkNoRequest();
				request1.setSysCode(sysCode);
				request1.setPklNo(plNo);
				BizResult<ParkingLotDTO> bizResult2 = parkingLotQueryService.QueryParkingLotByPkNo(request1);
				lotDTO = ResultUtils.getBizResultData(bizResult2);
			}
			if(lotDTO == null || StringUtils.isBlank(lotDTO.getPlName()) || lotDTO.getPlName().equals("null")){
				continue;
			}
			ParkLotStatisticVO lotStatisticVO = new ParkLotStatisticVO();
			lotStatisticVO.setPlNo(plNo);
			lotStatisticVO.setPlName(lotDTO.getPlName());
			lotStatisticVO.setFreeBerthsNum(freeBerthsNumMap.get(plNo));
			lotStatisticVO.setTotalBerthsNum(lotDTO.getPlBerthNum());
			lotStatisticVO.setUseingBerthsNum(lotDTO.getPlBerthNum()-freeBerthsNumMap.get(plNo));
			//设置地磁在线离线数量
			//TODO 临时方案，只有红山区有数据
			result.add(lotStatisticVO);
		}
		return new BizResultVO<List<ParkLotStatisticVO>>().setData(result);
	}
	
	private List<String> GetParkLotNosByCurrUser(String sysCode) throws Exception {
		logger.info("根据登录人权限获取停车场列表");
		List<String> plNos = Lists.newArrayList();
		//2.调用接口查询当前登录人管辖的停车场名称
		/*plNos.add("A320211000");
		plNos.add("B1504020C7");*/
		UserInfo userInfo = sessionCommUtil.getUserInfo();
		if(userInfo!=null){
			TdCompanyParkQueryRequest tdCompanyParkQueryRequest = new TdCompanyParkQueryRequest();
			tdCompanyParkQueryRequest.setCustIdList(userInfo.getOrgIds());
			tdCompanyParkQueryRequest.setSysCode(sysCode);
			tdCompanyParkQueryRequest.setSessionId(session.getId());
			BizResult<List<String>> bizResult = tdCustCompanyService.queryParkNoByCustIds(tdCompanyParkQueryRequest );
			plNos = ResultUtils.getBizResultData(bizResult);
		}else{
			throw new BizException(ErrorType.AUTH_TOKEN_NOT_EXISTS);
		}
		return plNos;
	}
}
