package com.zteits.irain.portal.web.parkinglotcloudplatform.datastatistic;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.dubbo.common.utils.CollectionUtils;
import com.alibaba.fastjson.JSON;

import com.clouds.common.utils.excle.ExcelUtil;
import com.clouds.common.utils.excle.ExcleFillDateManager;
import com.clouds.common.utils.excle.Layouter;
import com.clouds.common.web.BizController;
import com.clouds.common.web.vo.BizResultVO;
import com.clouds.common.web.vo.EasyUIDataGridVO;
import com.zteits.clouds.api.apibase.bean.BaseInfo;
import com.zteits.clouds.api.apibase.bean.BizResult;
import com.zteits.clouds.api.apibase.bean.PageBean;
import com.zteits.clouds.api.apibase.constants.ErrorType;
import com.zteits.clouds.api.apibase.exception.BizException;
import com.zteits.clouds.api.dto.clouds.dto.YearMonthCardStatisticDTO;
import com.zteits.clouds.api.dto.clouds.dto.YearMonthCardStatisticSumDTO;
import com.zteits.clouds.api.dto.pay.param.YearMonthCardStatisticRequest;
import com.zteits.clouds.api.service.clouds.YearMonthCardStatisticService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * 企业云平台->数据统计->年卡月卡统计。
 *
 * Copyright: Copyright (c) 2017  zteits
 *
 * @ClassName: InOutParkStatisticController.java
 * @Description:
 * @version: v1.0.0
 * @author: langlw
 * @date: 2017年7月26日   上午11:30:04
 * Modification History:
 * Date             Author          Version            Description
 * ---------------------------------------------------------*
 * 2017年7月26日      langlw          v1.0.0               创建
 */
@Api(value = "企业云平台->数据统计->年卡月卡统计", description = "企业云平台->数据统计->年卡月卡统计")
@RestController
@RequestMapping(value = "/yearMonthCardStatistic")
public class YearMonthCardStatisticController extends BizController {

    private static final Logger logger = LoggerFactory.getLogger(InOutParkStatisticController.class);

    @Value("${project.syscode}")
    private String sysCode;
    @Autowired
    private YearMonthCardStatisticService yearMonthCardStatisticService;

    @ApiOperation("年卡月卡统计")
    @PostMapping("queryYearMonthCardStatistic")
    @ResponseBody
    public BizResultVO<EasyUIDataGridVO<YearMonthCardStatisticDTO>> queryYearMonthCardStatistic(
        @RequestBody YearMonthCardStatisticRequest request) throws Exception {
        logger.info("调用DUBBO服务入参：" + JSON.toJSONString(request));
        if (null == request.getBeginTime() || null == request.getEndTime()) {
            throw new BizException(ErrorType.PARAMM_NULL, "开始时间和结束时间");
        }
        BizResult<PageBean<YearMonthCardStatisticDTO>> bizResult = yearMonthCardStatisticService
            .queryYearMonthCardStatistic(request);
        return returnJqGridData(bizResult, YearMonthCardStatisticDTO.class);
    }

    @ApiOperation("年卡月卡统计折线图")
    @PostMapping("queryYearMonthCardStatisticForChart")
    @ResponseBody
    public BizResultVO<List<YearMonthCardStatisticSumDTO>> queryYearMonthCardStatisticForChart(
        @RequestBody YearMonthCardStatisticRequest request) throws Exception {
        logger.info("调用DUBBO服务入参：" + JSON.toJSONString(request));
        if (null == request.getBeginTime() || null == request.getEndTime()) {
            throw new BizException(ErrorType.PARAMM_NULL, "开始时间和结束时间");
        }
        BizResult<List<YearMonthCardStatisticSumDTO>> bizResult = yearMonthCardStatisticService
            .queryYearMonthCardStatisticSum(request);
        logger.info("调用DUBBO服务返回结果：" + JSON.toJSONString(bizResult));
        return new BizResultVO<>(bizResult);
    }

    @ApiOperation("导出年卡月卡统计excel")
    @GetMapping("exportYearMonthStatisticExcel")
    public void exportYearMonthStatisticExcel(@RequestParam Long beginTime, @RequestParam Long endTime,
        @RequestParam List<String> parkIdList, @RequestParam String parkNames,
        Integer orderType, Integer payType, Integer dataState,
        HttpServletRequest requests, HttpServletResponse response) throws Exception {

        YearMonthCardStatisticRequest request = new YearMonthCardStatisticRequest();
        request.setSysCode(sysCode);
        request.setBeginTime(new Date(beginTime));
        request.setEndTime(new Date(endTime));

        request.setParkIdList(parkIdList);
        request.setBaseRequest(new BaseInfo(1, 0));
        request.setOrderType(orderType);
        request.setPayType(payType);
        request.setDataState(dataState);

        BizResult<PageBean<YearMonthCardStatisticDTO>> bizResult = yearMonthCardStatisticService
            .queryYearMonthCardStatistic(request);
        BizResult<List<YearMonthCardStatisticSumDTO>> bizResultSum = yearMonthCardStatisticService
            .queryYearMonthCardStatisticSum(request);

        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat format2 = new SimpleDateFormat("yyyy-MM-dd");
        String[] title = new String[] {"交易时间", "停车场", "类型", "车牌号", "支付方式", "支付金额", "有效期", "状态"};
        String sheetName = "年卡月卡统计";
        String fileName = "年卡月卡统计" + format2.format(new Date());

        String[] billTitle = new String[9];
         billTitle = new String[2];
        //if (!org.springframework.util.CollectionUtils.isEmpty(bizResultSum.getData())) {
        //    List<YearMonthCardStatisticSumDTO> dtoList = bizResultSum.getData();
        //    String amountsum = "";
        //    String sum = "";
        //    for (YearMonthCardStatisticSumDTO dto : dtoList) {
        //        amountsum = dto.getAmountSum() == null ? " " : dto.getAmountSum() + "," + amountsum;
        //        sum = dto.getOrderSum() + "," + sum;
        //    }
        //    String ordertype = "";
        //    if (null != orderType) {
        //        if (2 == orderType) {
        //            ordertype = "年卡";
        //        } else if (3 == orderType) {
        //            ordertype = "月卡";
        //        }
        //    }
        //    String datastate = "";
        //    if (null != dataState) {
        //        if (1 == dataState) {
        //            datastate = "有效";
        //        } else if (0 == dataState) {
        //            datastate = "无效";
        //        }
        //    }
        //
            billTitle[0] = "年卡月卡统计";
        //    billTitle[1] = "汇总";
        //    billTitle[2] = "停车场:  " + parkNames;
        //    billTitle[3] = "时间:  " + endTime;
        //    billTitle[4] = "类型:" + (orderType != null ? ordertype : "");
        //    billTitle[5] = "状态:  " + (dataState != null ? datastate : "");
        //    billTitle[6] = "支付方式:  " + (payType != null ? payType : "");
        //    billTitle[7] = "每个月的总笔数" + sum;
        //    billTitle[8] = "每个月的总金额数" + amountsum;
        //} else {
        //    billTitle[0] = "年卡月卡统计";
        //    billTitle[1] = "汇总";
        //    billTitle[2] = "停车场:  ";
        //    billTitle[3] = "时间:  ";
        //    billTitle[4] = "类型:";
        //    billTitle[5] = "状态:  ";
        //    billTitle[6] = "支付方式:  ";
        //    billTitle[7] = "每个月的总车数";
        //    billTitle[8] = "每个月的总金额数";
        //}

        // 1.创建excel信息
        XSSFSheet workSheet = ExcelUtil.createExcel(sheetName);
        // 2.设置excel表头和表体
        Layouter.buildReport(workSheet, title,0,0);
        // 3.填充数据
        List<Object[]> contentList = new ArrayList<Object[]>();
        List<YearMonthCardStatisticDTO> list = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(bizResult.getData().getDataList())) {
            list = bizResult.getData().getDataList();
        }
        for (YearMonthCardStatisticDTO e : list) {
            Object[] obj = new Object[title.length];
            int index = 0;
            obj[index++] = e.getPayFinishTime() != null ? format.format(e.getPayFinishTime()) : "";
            obj[index++] = e.getParkName();
            String cardType="";
            if (null != e.getOrderType()) {
                if (2 == e.getOrderType()) {
                    cardType = "年卡";
                }else{
                    cardType = "月卡";
                }
            }
            obj[index++]=cardType;
            obj[index++] = e.getCarNumber();
            if (e.getPayType() != null) {
                switch (e.getPayType()) {
                    case 1:
                        obj[index++] = "支付宝";
                        break;
                    case 2:
                        obj[index++] = "微信";
                        break;
                    case 3:
                        obj[index++] = "银联";
                        break;
                    case 4:
                        obj[index++] = "微信";
                        break;
                    default:
                        obj[index++] = "现金";
                        break;
                }
            } else {
                obj[index++] = "";
            }
            obj[index++] = e.getAmount() != null ? e.getAmount() : "0.00";
            obj[index++] = e.getEffDate() != null ? format.format(e.getEffDate())
                : "" + "-" + e.getExpDate() != null ? format.format(e.getExpDate()) : "";
            //有效、无效
            Date d = new Date();
            String state = "";
            if ((d.getTime() <= e.getExpDate().getTime()) && (d.getTime() >= e.getEffDate().getTime())) {
                state = "有效";
            } else {
                state = "无效";
            }
            obj[index++] = state;
            contentList.add(obj);
        }
        ExcleFillDateManager fillUserManager = new ExcleFillDateManager();
        fillUserManager.fillSalesOrga(workSheet, title, contentList, 2);
        // 4.excel输出配置
        ExcelUtil.write(response, workSheet, fileName);
    }

}
