package com.zteits.irain.portal.web.parkinglotcloudplatform.parklotmanage;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpSession;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.dubbo.common.utils.CollectionUtils;
import com.clouds.common.cache.park.ParkingLotCacheUtil;
import com.clouds.common.entity.UserInfo;
import com.clouds.common.utils.ResultUtils;
import com.clouds.common.web.SessionCommUtil;
import com.clouds.common.web.vo.BizResultVO;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.zteits.clouds.api.apibase.bean.BizResult;
import com.zteits.clouds.api.apibase.constants.ErrorType;
import com.zteits.clouds.api.apibase.exception.BizException;
import com.zteits.clouds.api.dto.park.dto.ParkingLotDTO;
import com.zteits.clouds.api.dto.park.param.QueryParkLotInfoByPkNoRequest;
import com.zteits.clouds.api.dto.pay.param.TdCompanyParkQueryRequest;
import com.zteits.clouds.api.service.park.ParkingLotQueryService;
import com.zteits.clouds.api.service.pay.TdCustCompanyService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 停车场管理
 * 
 * Copyright: Copyright (c) 2017  zteits
 * 
 * @ClassName: ParkingLotController.java
 * @Description: 
 * @version: v1.0.0
 * @author: zhaowg
 * @date: 2017年6月16日   下午5:27:21 
 * Modification History: 
 * Date             Author          Version            Description
 *---------------------------------------------------------*
 * 2017年6月16日      zhaowg              v1.0.0               创建
 */
@Api(value="停车场管理",description="停车场管理")
@RestController
@RequestMapping("parkLotManage")
public class ParkLotManageController{
	private Logger logger = LoggerFactory.getLogger(ParkLotManageController.class);
	@Autowired
	private HttpSession session;
	@Autowired
	private SessionCommUtil sessionCommUtil;
	@Autowired
	private TdCustCompanyService tdCustCompanyService;
	@Autowired
	private ParkingLotQueryService parkingLotQueryService;
	/**
	 * 根据登录人权限获取停车场名称列表
	 * @return
	 * 2017年6月16日 zhaowg
	 * @throws Exception 
	 */
	@ApiOperation("根据登录人权限获取停车场名称列表")
	@GetMapping("getParkLotNamesByUserId")
	public BizResultVO<List<Map<String,String>>> GetParkLotNameListByUserId(@RequestParam String sysCode) throws Exception {
		List<Map<String,String>> result = Lists.newArrayList();
		//拥有的停车场编号
		List<String> plNos = this.GetParkLotNosByCurrUser(sysCode);
		//查询对应的停车场信息
		if(CollectionUtils.isEmpty(plNos)){
			return new BizResultVO<List<Map<String,String>>>().setData(result);
		}
		for (String plNo : plNos) {
			Map<String,String> plNoMap = Maps.newHashMap();
			//先查询缓存
			ParkingLotDTO lotDTO = ParkingLotCacheUtil.getParkLotByPlNo(plNo);
			if(lotDTO == null){
				//查询数据库
				QueryParkLotInfoByPkNoRequest request = new QueryParkLotInfoByPkNoRequest();
				request.setSysCode(sysCode);
				request.setPklNo(plNo);
				BizResult<ParkingLotDTO> bizResult2 = parkingLotQueryService.QueryParkingLotByPkNo(request);
				lotDTO = ResultUtils.getBizResultData(bizResult2);
			}
			if(lotDTO == null || StringUtils.isBlank(lotDTO.getPlName()) || lotDTO.getPlName().equals("null")){
				continue;
			}
			plNoMap.put("code",plNo );
			plNoMap.put("name",lotDTO.getPlName());
			result.add(plNoMap);
		}
		
		return new BizResultVO<List<Map<String,String>>>().setData(result);
	}
	
	private List<String> GetParkLotNosByCurrUser(String sysCode) throws Exception {
		logger.info("根据登录人权限获取停车场列表");
		UserInfo userInfo = sessionCommUtil.getUserInfo();
		List<String> plNos = userInfo.getOrgIds();

		return plNos;
	}
}
